#include "stdafx.h"
#include "TsBuffer.h"

CTsBuffer::CTsBuffer(IEventHandler *pEventHandler)
	: CMediaDecoder(pEventHandler, 1, 1)
	, m_bytePacket( TS_PACKETSIZE * TS_PACKET_BUFFER )
	, m_uWrite( 0 )
	, m_uRead( 0 )
	, m_uPacketCount( 0 )
{
	this->m_hEvent = ::CreateEvent( NULL, FALSE, FALSE, NULL );
	Reset();
}

CTsBuffer::~CTsBuffer()
{
	::CloseHandle( this->m_hEvent );
}

const bool CTsBuffer::InputMedia(CMediaData *pMediaData, const DWORD dwInputIndex)
{
	CTsPacket *pTsPacket = static_cast<CTsPacket *>(pMediaData);
	this->PushPacket( pTsPacket->GetData() );
	return true;
}

void CTsBuffer::Reset()
{
	::ResetEvent( this->m_hEvent );
	this->FillBuffer();
}

void CTsBuffer::PushPacket( BYTE* t_byteBuffer )
{
	CBlockLock Lock(&m_DecoderLock);
	if( this->m_uWrite >= TS_PACKET_BUFFER )
	{
		this->m_uWrite = 0;
	}
	::memcpy( &this->m_bytePacket[ this->m_uWrite * TS_PACKETSIZE ], t_byteBuffer, TS_PACKETSIZE );
	this->m_uWrite++;
	this->m_uPacketCount++;

	::SetEvent( this->m_hEvent );
}

BYTE* CTsBuffer::PopPacket(void)
{
	CBlockLock Lock(&m_DecoderLock);
	if( this->m_uRead >= TS_PACKET_BUFFER )
	{
		this->m_uRead = 0;
	}

	if( this->m_uPacketCount < 1 )
	{
		return NULL;
	}

	BYTE* t_bytePacket = &this->m_bytePacket[ this->m_uRead * TS_PACKETSIZE ];
	this->m_uRead++;
	this->m_uPacketCount--;

	return t_bytePacket;
}

BYTE* CTsBuffer::PopPacketsBlock(void)
{
	CBlockLock Lock(&m_DecoderLock);
	if( this->m_uRead >= TS_PACKET_BUFFER )
	{
		this->m_uRead = 0;
	}

	if( this->m_uPacketCount < TS_PACKET_BLOCK )
	{
		return NULL;
	}

	BYTE* t_bytePacket = &this->m_bytePacket[ this->m_uRead * TS_PACKETSIZE ];
	this->m_uRead += TS_PACKET_BLOCK;
	this->m_uPacketCount -= TS_PACKET_BLOCK;

	return t_bytePacket;
}

void CTsBuffer::FillBuffer(void)
{
	CBlockLock Lock(&m_DecoderLock);
	::memset( &this->m_bytePacket[ 0 ], 0, this->m_bytePacket.size() );
	this->m_uWrite = 0;
	this->m_uRead = 0;
	this->m_uPacketCount = 0;
}

UINT32 CTsBuffer::GetPacketCount(void)
{
	CBlockLock Lock(&m_DecoderLock);
	return this->m_uPacketCount;
}

HANDLE CTsBuffer::GetWriteEvent(void)
{
	return this->m_hEvent;
}
